#####################################################################################################################
#Copyright IBM
#
# This PERL script was created by Shirley Dango of Rational Software.
# It was created on November 18, 2002 for the RATLC production database as part of the CC/CQ/UCM Integration project.
# The purpose is to create the Build record in the RATLC production database.
# The user must provide the user login id, password, database name (RATLC), database set name,
# pvob, build type, product, version and new baseline to create. 
######################################################################################################################
# 
# Updated by: Shirley Dango
# Updated on: April 21, 2003 
# Updated by Katie Carpenter, October & November 2007
# Updated by Stuart Poulin Nov 2007
#
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user			The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname			The schema repository connection to the database
#	dbset			The user database connection
#	pvob			The name of the project vob
#	baseline		The name of the baseline
#	build			The name of the build
#	buildstatus		The type of build 
#	projectid		The CQ id of the project this build is related to - for example ALM0000045
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
# thl3   RATLC00475598       10/19/2005      THL               SQUID Scripts for baseline and build create fail when 
#                                                              login is ldap enabled
# thl4   RATLC00475646       10/19/2005      THL               Need to check for "\" on pvob parm 
# thl5   RATLC00475658       10/23/2005      THL               SQUID Script create_build_record allows multiple build 
#                                                              records to be created with the same build name 
#                                                              (Build field value)
#			     10/11/2007	     katherns	       Modify references of 'Build' to 'BTBuild'
#
#        RATLC00396109       10/26/2007      katherns          build cannot be created against an obsolete baseline
#
#        RATLC00396140       10/26/2007      katherns          build record does not need to contain activities
#
#        RATLC00396143       10/26/2007      katherns          change references of 'relCWActivities' to 'Activities'
#
#        RATLC01214353       10/26/2007      katherns          add Build_System_ID as a parameter (build)
#
#        RATLC01216069       11/5/2007       katherns          add ALMBuildType as a parameter
#
#        RATLC00396394       1/14/2008       katherns          Update scripts for M3 release.
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict;
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use CQPerlExt;
use Env;
use IO::File;
use LWP::Simple;
use POSIX;
use Time::Local;
my ($entity);
my $id;
my ($failure);
my ($baselineentity);
my $result;
my $progname = basename $0;
my $status ;
my $buildstatus ; #thl2
my $userLogin ; #thl2
my ($resultset,$resultsetstatus,$buildId);
my $recordOwner;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $result = &main();

if ($result) {
    print "ID $id\n";
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

####
# Subroutine to display the usage
sub Usage {
	print "Utility to create a build record using the CQ API
    Usage:
       $progname -test
         -help 
         -user <username>
         -pw <password>
         -dbname <database name> 
         -dbset <database set> 
         -pvob <pvob> 
         -baseline <baseline> 
         -build <build name> 
         -buildstatus <buildstatus>
         -buildtype <type of build> 
         -projectid <project id> 
	 -logfile <logfile> 
"		
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s
    pw=s
    dbname=s
    dbset=s
    pvob=s
    baseline=s
    build=s
    buildstatus=s
    buildtype=s
    projectid=s
    owner=s
    logfile=s
    test
    help
   
);


my %opts = (
    user => undef,
    pw => undef,
    dbname => undef,
    dbset => undef,
    pvob => undef,
    baseline => undef,
    build =>undef,
    buildstatus => undef,
    buildtype => undef,
    projectid => undef,
    owner => undef,
    logfile => undef,
);


####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
 	$logger->log_print_message("Printing Usage to the command line:");	
      	my $retval = &Usage;
      	return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "Could not create the log file directory: $logdir");
            		$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "Could not open the log file: $opts{logfile}.");
        	$logger->log_print_error( __LINE__ . "Script execution will continue without it.");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("Logging started at $log_time"); 
}	 

####
# Display the help usage
if ($opts{help}) {
	print "-user		The login id of the ClearQuest user.\n";
	print "-pw		The password of the ClearQuest user.\n";
	print "-dbname		The schema repository connection to the database.\n";
	print "-dbset		The user database connection.\n";
	print "-pvob		The name of the project vob.\n";
	print "-baseline	The name of the baseline.\n";
	print "-build		The name of the build.\n";
	print "-buildstatus	The status of the build.\n";
        print "-buildtype       The type of build.\n";
	print "-projectid	The id of the project this build is related to.\n";
	print "-logfile		This is an optional flag to define the log file path. Default prints to console.\n";
	print "-test		This is an optional flag that can be set for testing file inputs.\n";
	print "-help		This is an optional flag for the help.\n";
    return 1;
}

# Display test inputs to log file 03/26/03
if ($opts{test}){
	$logger->log_print_message(" Defined user inputs:");
	$logger->log_print_message(" Defined user login is $opts{user}.");
	$logger->log_print_message(" Defined dbset is $opts{dbset}.");
	$logger->log_print_message(" Defined database name is $opts{dbname}.");
	$logger->log_print_message(" Defined baseline name is $opts{baseline}.");
	$logger->log_print_message(" Defined Project VOB is $opts{pvob}.");
	$logger->log_print_message(" Defined Build Status is $opts{buildstatus}.");
	$logger->log_print_message(" Defined Build Type is $opts{buildtype}.");
	$logger->log_print_message(" Defined Project ID is $opts{projectid}.");
	if ($opts{logfile}){ $logger->log_print_message("Defined logfile is $opts{logfile}.");}
	return 1;
}

####
# End program if any of the required parameters are not defined
if (!defined $opts{user}
  or !defined $opts{pw}
  or !defined $opts{dbname}
  or !defined $opts{dbset}
  or !defined $opts{pvob}
  or !defined $opts{baseline}
  or !defined $opts{buildstatus}
  or !defined $opts{buildtype}
  or !defined $opts{projectid}
  or !defined $opts{build}) {
  	$logger->log_print_message(" Printing Usage to the command line:");
 	my $retval = &Usage();
 	return 0;
  }
;


###
# Start a CQ session
my $session = CQPerlExt::CQSession_Build();

# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
# Logon to the specified database
eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
$logger->log_print_error( __LINE__ . " Login to $opts{dbname} failed: $@"); 
return 0;
}
$userLogin = $session->GetUserLoginName() ; #thl3
$logger->log_print_message(" Login to $opts{dbname} $userLogin was successful."); #thl3


# See if the build record already exists and return if it does            thl5
my $queryDef = $session->BuildQuery("BTBuild");                            #thl5   
$failure = $queryDef->BuildField("id");                                  #thl5
my $operator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND); #thl5 
$operator->BuildFilter("OldID", $CQPerlExt::CQ_COMP_OP_EQ, ["$opts{build}"]); #thl5
eval {$resultset = $session->BuildResultSet($queryDef)};                  #thl5
if ($@) {                                                                 #thl5
   $logger->log_print_error(__LINE__ . ":  Building Query Failed: $@");   #thl5
   return 0;                                                              #thl5
}                                                                         #thl5
else {                                                                    #thl5
   $resultset->Execute();                                                 #thl5
   $resultsetstatus = $resultset->MoveNext;                               #thl5
   if ($resultsetstatus == $CQPerlExt::CQ_SUCCESS) {                      #thl5
      $buildId = $resultset->GetColumnValue(1);
      $logger->log_print_error(__LINE__ . ":  Build Record $opts{build} already exists as $buildId ");#thl5
      return 0;                                                           #thl5
   }                                                                      #thl5
}                                                                         #thl5

# Find the Baseline record in the ClearQuest database

my $baselineentity = $session->GetEntity("ALMBaseline", $opts{pvob}.' '.$opts{baseline}); 


$logger->log_print_message(" Baseline entity was found for $opts{baseline} in project VOB: $opts{pvob}.");

# Determine if the Baseline record was found in ClearQuest database
if ($baselineentity){

#Do not create build if baseline is obsolete
my $obsoletevalue = $baselineentity->GetFieldValue("Obsolete")->GetValue();
if($obsoletevalue eq 'Yes'){
$logger->log_print_message(" The Baseline Record is Obsolete. Build Record not created.");
$logger->log_print_error(" The Baseline Record is Obsolete. Build Record not created.");
return 0;
}

	$logger->log_print_message(" Creating new Build record.");
	
	## Build the Build record in ClearQuest database
	my $entity;
	eval {$entity = $session->BuildEntity("BTBuild")};
	if ($@) {
		$logger->log_print_error( __LINE__ . " Can't open build record:"); 
		$logger->log_print_error( __LINE__ . " $@");
        return 0;
	}


	my $baselinename = $opts{pvob} . ' ' . $opts{baseline};
	
	## Set field values for pvob, baseline, buildstatus,project, and version
	
	$entity->SetFieldValue("ALMProject", $opts{projectid});
	$entity->SetFieldValue("Build_System_ID", $opts{build});	
	$entity->SetFieldValue("ALMBuildStatus",$opts{buildstatus});
        $entity->SetFieldValue("ALMBuildType", $opts{buildtype});
	$entity->SetFieldValue("ALMOwner", $userLogin); 
        $entity->SetFieldValue("ALMBaseline", $baselinename); 
			
	## New Build ID
	$id = $entity->GetFieldValue("id")->GetValue();
	$logger->log_print_message("ID=$id");
	$logger->log_print_message(" Validating new Build record.");
	$status = "" ; #thl2
	eval {$status = $entity->Validate()};
	if (($@) || ($status)) {           #thl2
		$entity->Revert();
		$logger->log_print_error(__LINE__ . " Couldn't validate build record in ClearQuest Database.");
		$logger->log_print_error(__LINE__ . "  eval=$@");
                $logger->log_print_error(__LINE__ . "  Returned Status is $status."); #thl2
		return 0;
	}
        $status = "" ; #thl2
	$logger->log_print_message(" Committing new Build record.");
	eval {$status = $entity->Commit()}; #thl2
	if (($@) || ($status)) {            #thl2
		$entity->Revert();
		$logger->log_print_error( __LINE__ . " Couldn't commit build record in ClearQuest Database.");
		$logger->log_print_error( __LINE__ . " $@");
                $logger->log_print_error(__LINE__ . "  Returned Status is $status."); #thl2
		CQPerlExt::CQSession_Unbuild($session);
		return 0;
	}
        # Write to log file
	$logger->log_print_message(" The new build record has been created.");
	$entity = undef;
	CQPerlExt::CQSession_Unbuild($session);
	} 
else {
	# Write to log file 
	$logger->log_print_message(" Baseline record not found. Please create a new Baseline record.");
	$logger->log_print_message(" No Build record created.");	
	CQPerlExt::CQSession_Unbuild($session);
	return 0;
} # end of creating Build and updating Baseline record in CQ
# Write to log file

$logger->log_print_success("Finished create_build_record script.");
$logger->log_print_message(" create_build_record.pl script finished");
CQPerlExt::CQSession_Unbuild($session);

return 1;
}
#### SUBROUTINES FOR PROGRAM
