#####################################################################################################################
# Copyright IBM
#
# This PERL script was created by Shirley Dango of Rational Software.
# It was created on December 6, 2002 for the RATLC production database as part of the CC/CQ/UCM Integration project.
# The purpose is to update the Build record in the RATLC production database.
# The script will search for the build record within the RATLC production database.
# The user must provide the user login id, password, database name (RATLC), database set name,
# the build record to update by using the build name or build ID, and the new promotion level information.
#
# Update note: This script currently has an optional parameter for the build start time.  At this time, the record
# cannot be updated with this information.  If the parameter is used and a value passed in, the record
# is not updated with the information. 
######################################################################################################################
# 
# 
# Updated by: Shirley Dango
# Updated on: April 21, 2003
# Updated January 2008, by Katie Carpenter
#
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user			The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname			The schema repository connection to the database
#	dbset			The user database connection
#	pvob			The name of the project vob
#	baseline		The name of the baseline
#	id			The id of the BTBuild record to update
#	buildstatus		Specifies what the status of the build.  If it passed or failed.
#	buildstarttime		Time of day build was started.
#	buildendtime		Time of day build completed.
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
#        RATLC00396394       1/14/2008       katherns          Update scripts for M3 release.
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use IO::File;
use LWP::Simple;
use POSIX;
use Time::Local;
# use Date::Manip qw(ParseDate UnixDate);

my $progname = basename $0;
my ($entity);
my ($failure,$status); #thl2
my $CRList;
my $item;
my $slashvob;
my $noslashvob;
my $buildtype;
my $securitypolicy;

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $result = &main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

####
# Subroutine to display the usage
sub Usage {
    print "

    Utility to logon and create a session using the CQ API
    Usage:
        $progname 
 	-id RATLC000xxxxx
 	-user <username> 
 	-pw <password> 
 	-dbname <database name> 
 	-dbset <database set> 
 	-pvob<pvob>
 	-baseline<baseline> 
	-buildstatus<buildstatus>
	-securitypolicy<ALM security policy>
	-buildtype<build type>    
    -buildstarttime<buildstarttime>
    -buildendtime<buildendtime>   
    -logfile
    -test
    -help		
";
return 0;
}

sub main {	
####
# Define the input parameters
my @opts = qw(
    id=s
    user=s	
    pw=s		
    dbname=s
    dbset=s
    pvob=s
    baseline=s
    buildstatus:s
    securitypolicy=s
    buildtype=s
    buildstarttime:s
    buildendtime:s
    state=s
    logfile=s
    test
    help
);

my %opts = (
    id => undef,
    user => undef, 
    pw => undef,
    dbname => undef,
    dbset => undef,
    pvob => undef,
    baseline => undef,
    securitypolicy=> undef,
    buildtype=> undef,
    logfile =>undef,
    state => "Modify", 
);

####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
    $logger->log_print_message(__LINE__ . " Printing Usage to the command line:");	
     my $retval = &Usage;
     return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	$opts{logfile} =~ /(.*)\\.*$/;
	my $logdir = $1;
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error(__LINE__ . "Could not create the log file directory: $logdir");
            		$logger->log_print_error(__LINE__ . "Script execution will continue without it.");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error(__LINE__ . "Could not open the log file: $opts{logfile}.");
        	$logger->log_print_error(__LINE__ . "Script execution will continue without it.");
   	 }
        $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("Logging started at $log_time"); 
}	 

####
# Display the help usage
if ($opts{help}) {
 
   	print "Required parameters listed below: \n";     
        print "-user  		The login id of the ClearQuest user\n";
	print "-pw		The password of the ClearQuest user\n";
        print "-dbname		The schema repository connection to the database\n";
	print "-dbset		The user database connection\n";
	print "-logfile		The path where the log file will be sent\n";
        print "-pvob		The name of the project vob\n";
	print "-baseline  	The name of the baseline\n";
	print "-securitypolicy	The ALM Security Policy to assign to the record\n";
	print "-buildtype  	The the type of build \n";
	print "Optional parameters listed below: \n";
	print "-state <state>   The state you wish to promote the build record to\n";	
	print "-buildstatus		Specifies what the status of the build.  If it passed or failed.\n";
	print "-buildstarttime		Time of day build was started.\n";
	print "-buildendtime		Time of day build completed.\n";
	print "-test		This is an optional flag that can be set for testing file inputs.\n";
	print "-help		This is an optional flag for the help.\n";
    return 1;
}

# Display test inputs to log file 
if ($opts{test}){
$logger->log_print_message(__LINE__ . " Defined user inputs:");
	$logger->log_print_message(__LINE__ . " Defined Build Record ID $opts{id}.");
	$logger->log_print_message(__LINE__ . " Defined user login is $opts{user}.");
	$logger->log_print_message(__LINE__ . " Defined dbset is $opts{dbset}.");
	$logger->log_print_message(__LINE__ . " Defined database name is $opts{dbname}.");
	$logger->log_print_message(__LINE__ . " Defined baseline name is $opts{baseline}.");
	$logger->log_print_message(__LINE__ . " Defined state is $opts{state}.");
	$logger->log_print_message(__LINE__ . " Defined Project VOB is $opts{pvob}.");
	$logger->log_print_message(__LINE__ . " Defined Build Type is $opts{buildtype}.");
	$logger->log_print_message(__LINE__ . " Defined Product is $opts{product}.");
	$logger->log_print_message(__LINE__ . " Defined Security Policy is $opts{securitypolicy}.");
	if ($opts{logfile}){ $logger->log_print_message("Defined logfile is $opts{logfile}.");}
	if ($opts{buildstatus}){ $logger->log_print_message("Defined Build Status is $opts{buildstatus}.");}
	if ($opts{buildstarttime}){ $logger->log_print_message("Defined Build Start Time is $opts{buildstarttime}.");}
	if ($opts{buildendtime}){ $logger->log_print_message("Defined Build End Time is $opts{buildendtime}.");}
	return 1;
}
####
# End program if any of the required parameters are not defined


if (!defined $opts{user}
  or !defined $opts{pw} 
  or !defined $opts{dbname} 
  or !defined $opts{dbset} 
  or !defined $opts{id}
  or !defined $opts{pvob} 
  or !defined $opts{securitypolicy}
  or !defined $opts{buildtype}) {
  $logger->log_print_message(__LINE__ . " Printing Usage to the command line:");
  my $retval = &Usage();
  return 0;
}
;
####
# Start a CQ session
print ("Starting CQ Session\n") ;
my $session = CQPerlExt::CQSession_Build();
####
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
# Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
$logger->log_print_error(__LINE__ . "main: Login to $opts{dbname} failed: $@"); 
return 0;
}

$logger->log_print_message(__LINE__ . " Login to $opts{dbname} successful.");


#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}
($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});

#### Build a query to find the Build record based upon Build ID
my $record;
eval {$record = $session->GetEntity("BTBuild", $opts{id})};
if ($@) {
	$logger->log_print_error(__LINE__ . "main: No records found to match Build Record ID $opts{id} "); 
	$logger->log_print_error(__LINE__ . "main: Please run create_build_record.pl to create desired build record."); 
		return 0;
}
	{
	#### Print out the resultset
	$logger->log_print_message(__LINE__ . " Found record: $opts{id}");
	
	####Get record 
	# Add updates to the record
	$session->EditEntity($record,"$opts{state}");
	$logger->log_print_message(__LINE__ . " $opts{state} Record: $opts{id}");
	 	
        #Set required values

	my $baselinename = $noslashvob . ' ' . $opts{baseline};
	
	$record->SetFieldValue("ALMBaseline", $baselinename);
		$logger->log_print_message(__LINE__ . " Setting ALMBaseline: $baselinename");
	$record->SetFieldValue("ALMBuildType", $opts{buildtype});
		$logger->log_print_message(__LINE__ . " Setting ALMBuildType: $opts{buildtype}");
	$record->SetFieldValue("ALMOwner", $opts{user});
		$logger->log_print_message(__LINE__ . " Setting ALMOwner: $opts{user}");
	$record->SetFieldValue("ALMSecurityPolicy", $opts{securitypolicy});
		$logger->log_print_message(__LINE__ . " Setting ALMSecurityPolicy: $opts{securitypolicy}");	

	#Results from Building
	if ($opts{buildstatus}ne ""){
		$record->SetFieldValue("ALMBuildStatus", $opts{buildstatus});
		$logger->log_print_message(__LINE__ . " Setting BuildStatus: $opts{buildstatus}");
	}
	
	if ($opts{buildstarttime} ne ""){
		$record->SetFieldValue("Start_DateTime", $opts{buildstarttime});
		$logger->log_print_message(__LINE__ . " Setting BuildStartTime: $opts{buildstarttime}");
	}
	if ($opts{buildendtime} ne ""){
		$record->SetFieldValue("End_DateTime", $opts{buildendtime});
		$logger->log_print_message(__LINE__ . " Setting BuildEndTime: $opts{buildendtime}");
	}
		
	
	
	$logger->log_print_message(__LINE__ . " Validating the Build record $opts{id}");
	$status = "" ; #thl2
	eval {$status = $record->Validate()};
	if (($@) || ($status)) {  #thl2
		$logger->log_print_error(__LINE__ . " Couldn't validate build record in ClearQuest Database: $status");
	}
	if ($@) {
		$record->Revert();
		$logger->log_print_error(__LINE__ . "main: Couldn't validate build record in ClearQuest Database.");
		$logger->log_print_error(__LINE__ . "main: $@");
                $logger->log_print_error(__LINE__ . "Returned Status is: $status"); #thl2
		return 0;
	}
	$status = "" ; #thl2
	$logger->log_print_message(__LINE__ . " Committing the Build record $opts{id}");
	eval {$status = $record->Commit()};
	if (($@) || ($status)) {  #thl2
		$record->Revert();
		$logger->log_print_error(__LINE__ . "main: Couldn't commit build record in ClearQuest Database.");
		$logger->log_print_error(__LINE__ . "main: $@");
                $logger->log_print_error(__LINE__ . "Returned Status is: $status"); #thl2
		CQPerlExt::CQSession_Unbuild($session);
		return 0;
	}
	$record = undef;
	$logger->log_print_success(__LINE__ . " $opts{id} has been successfully updated.");
	CQPerlExt::CQSession_Unbuild($session);
	return 1;
	} #end block
$logger->log_print_success("Finished Update_build_record script.");
$logger->log_print_message(__LINE__ . " Update_build_record script complete");
CQPerlExt::CQSession_Unbuild($session);
return 1;
} #end sub main
#### End of Script
#### SUBROUTINES FOR PROGRAM
